{% extends 'djiffy/base.html' %}

{% block page-subtitle %}IIIF Manifests | {% endblock %}

{% block content %}
<div class="container djifbooks">
<h1>Manifests</h1> {# how to override label to make more meaningful, e.g. books ? #}
{% for manifest in manifests %}
    {% include 'djiffy/snippets/manifest_card.html' %}
{% endfor %}
</div>
{% endblock %}